<?php

/**
 * @file
 * Contains Drupal Commerce specific hook implementations.
 */

/**
 * Implements hook_commerce_payment_method_info().
 */
function commerce_liqpay_commerce_payment_method_info() {
  $payment_methods['liqpay'] = array(
    'base' => 'commerce_liqpay',
    'title' => t('LiqPay'),
    'terminal' => FALSE,
    'offsite' => TRUE,
    'active' => TRUE,
    'offsite_autoredirect' => TRUE,
    'callbacks' => array(
      'settings_form' => 'commerce_liqpay_payment_method_settings_form',
      'redirect_form' => 'commerce_liqpay_payment_method_redirect_form',
      'submit_form' => 'commerce_liqpay_payment_method_submit_form',
      'submit_form_validate' => 'commerce_liqpay_payment_method_submit_form_validate',
      'submit_form_submit' => 'commerce_liqpay_payment_method_submit_form_submit',
    ),
  );

  return $payment_methods;
}

/**
 * Returns the default settings for the LiqPay payment method.
 */
function commerce_liqpay_default_settings() {
  return array(
    'sandbox' => FALSE,
    'public_key' => '',
    'private_key' => '',
    'messages' => array(),

    // Additional fields for checkout review form.
    'checkout' => array(
      'logo' => TRUE,
      'description' => t('You can pay with Visa/MasterCard cards, cash (self-service terminal) or Privat24 internet-bank.'),
    ),

    // This description will be process with token_replace() and translate
    // before payments redirect. We need to keep this setting in english.
    // As result we should not apply t() or commerce_i18n_string() here.
    'description' => 'Order #[commerce-order:order-id] at [site:name].',
  );
}

/**
 * Implements CALLBACK_commerce_payment_method_settings_form().
 */
function commerce_liqpay_payment_method_settings_form($settings = array()) {
  // Apply default settings.
  $settings = (array) $settings + commerce_liqpay_default_settings();

  $form['sandbox'] = array(
    '#title' => t('Sandbox mode'),
    '#description' => t('Enables the testing environment for developers. Card will not be credited during payment process. All test payments will have `sandbox` status - success test payment.'),
    '#default_value' => $settings['sandbox'],
    '#type' => 'checkbox',
  );

  $form['public_key'] = array(
    '#title' => t('Public key'),
    '#description' => t('Key is the unique store identifier. You can get the key in !link.', array(
      '!link' => l(t('the store settings'), 'https://www.liqpay.com/en/admin/business'),
    )),
    '#default_value' => $settings['public_key'],
    '#type' => 'textfield',
    '#required' => TRUE,
  );

  $form['private_key'] = array(
    '#title' => t('Private key'),
    '#description' => t('Key is the unique store identifier. You can get the key in !link.', array(
      '!link' => l(t('the store settings'), 'https://www.liqpay.com/en/admin/business'),
    )),
    '#default_value' => $settings['private_key'],
    '#type' => 'textfield',
    '#required' => TRUE,
  );

  $form['description'] = array(
    '#type' => 'textfield',
    '#title' => t('Payment description'),
    '#description' => t('Payment description required by Liqpay. This description will be available in LiqPay interface. You can use token replacement patterns for adding order id, site name, etc.'),
    '#default_value' => $settings['description'],
    '#required' => TRUE,
  );

  $form['token_tree_link'] = array(
    '#theme' => 'token_tree_link',
    '#token_types' => array('commerce-order'),
  );

  $form['checkout'] = array(
    '#type' => 'fieldset',
    '#title' => t('Checkout form'),
    '#description' => t('These settings will affect descriptions for payment methods selection form.'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  $form['checkout']['logo'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add Liqpay logo'),
    '#default_value' => $settings['checkout']['logo'],
  );

  $form['checkout']['description'] = array(
    '#title' => t('Additional description'),
    '#default_value' => $settings['checkout']['description'],
    '#type' => 'textarea',
    '#rows' => 2,
  );

  $form['messages'] = array(
    '#type' => 'fieldset',
    '#title' => t('Messages'),
    '#description' => t('You can find more details about LiqPay statuses at !link.', array(
      '!link' => l(t('API reference'), 'https://www.liqpay.com/en/doc/checkout'),
    )),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  $form['messages']['i18n_help'] = array(
    '#markup' => t('If you need to translate those messages at multilingual site you should enable <em>String translation (i18n_string)</em> module that is a part of Internationalization project.'),
  );

  foreach (commerce_liqpay_transaction_statuses() as $status_name => $status) {
    $settings['messages'] += array(
      $status_name => $status['message'],
    );
    $form['messages'][$status_name] = array(
      '#type' => 'textarea',
      '#rows' => 2,
      '#title' => $status['title'],
      '#description' => t('API status machine name: !status', array(
        '!status' => $status_name,
      )),
      '#default_value' => $settings['messages'][$status_name],
    );
  }

  return $form;
}

/**
 * Implements CALLBACK_commerce_payment_method_submit_form().
 */
function commerce_liqpay_payment_method_submit_form($payment_method, $pane_values, $checkout_pane, $order) {
  if (!empty($payment_method['settings']['checkout']['logo'])) {
    $form['logo'] = array(
      '#theme' => 'image',
      '#path' => drupal_get_path('module', 'commerce_liqpay') . '/images/logo.png',
      '#alt' => t('LiqPay'),
      '#attributes' => array(
        'class' => array('liqpay-logo'),
      ),
    );
  }

  $form['description'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'class' => array('liqpay-description'),
    ),
    'markup_text' => array(
      '#markup' => $payment_method['settings']['checkout']['description'],
    ),
  );

  return $form;
}

/**
 * Implements CALLBACK_commerce_payment_method_submit_form_submit_validate().
 */
function commerce_liqpay_payment_method_submit_form_validate($payment_method, $pane_form, $pane_values, $order, $form_parents = array()) {
  // Return an error if the enabling action's settings haven't been configured.
  foreach (array('public_key', 'private_key', 'description') as $key) {
    if (empty($payment_method['settings'][$key])) {
      drupal_set_message(t('LiqPay service is not configured for use. Please contact an administrator to resolve this issue.'), 'error');
      return FALSE;
    }
  }

  return TRUE;
}

/**
 * Implements CALLBACK_commerce_payment_method_submit_form_submit().
 */
function commerce_liqpay_payment_method_submit_form_submit($payment_method, $pane_form, $pane_values, $order, $charge) {
  $transaction = commerce_payment_transaction_new('liqpay', $order->order_id);
  $transaction->instance_id = $payment_method['instance_id'];
  $transaction->amount = $charge['amount'];
  $transaction->currency_code = $charge['currency_code'];
  $transaction->status = COMMERCE_PAYMENT_STATUS_PENDING;
  commerce_payment_transaction_save($transaction);
}

/**
 * Implements CALLBACK_commerce_payment_method_redirect_form().
 */
function commerce_liqpay_payment_method_redirect_form($form, &$form_state, $order, $payment_method) {
  $data = commerce_liqpay_payment_data_encode($order, $payment_method['settings']);
  $signature = commerce_liqpay_generate_signature($data, $payment_method['settings']['private_key']);

  $form['data'] = array(
    '#type'  => 'hidden',
    '#value' => $data,
  );

  $form['signature'] = array(
    '#type'  => 'hidden',
    '#value' => $signature,
  );

  $form['submit'] = array(
    '#type'  => 'submit',
    '#value' => t('Go to LiqPay service'),
  );

  $form['#action'] = COMMERCE_LIQPAY_CHECKOUT_URL;

  return $form;
}
