<?php

/**
 * @file
 * Page callbacks for LiqPay integrations.
 */

/**
 * Page callback function.
 *
 * This page will receive API notifications of payment status change.
 * This is a server-server communication page.
 * See https://www.liqpay.com/en/doc/callback for more details.
 */
function commerce_liqpay_page_server() {
  if ($data = commerce_liqpay_post_validate()) {
    commerce_liqpay_process_transaction($data);
  }
}

/**
 * Page callback function.
 *
 * Payer will be redirected to this page after the payment.
 */
function commerce_liqpay_page_result() {
  // Check base params that should be exist.
  if (empty($_GET['order_id']) || empty($_GET['redirect_key'])) {
    return MENU_ACCESS_DENIED;
  }

  // Check order access.
  $order = commerce_order_load($_GET['order_id']);
  if (empty($order) || ($_GET['redirect_key'] != $order->data['payment_redirect_key'])) {
    return MENU_ACCESS_DENIED;
  }

  // Build checkout paths.
  $path_back = 'checkout/' . $order->order_id . '/payment/back/' . $order->data['payment_redirect_key'];
  $path_next = 'checkout/' . $order->order_id . '/payment/return/' . $order->data['payment_redirect_key'];

  // Load commerce transaction object.
  $transaction_conditions = array(
    'payment_method' => 'liqpay',
    'order_id' => $order->order_id,
  );
  $transactions = commerce_payment_transaction_load_multiple(array(), $transaction_conditions);
  if (empty($transactions)) {
    drupal_set_message(t('Something went wrong during payment. Please contact site administrator if you have any questions about your payment.'));
    drupal_goto($path_back);
  }
  $transaction = reset($transactions);

  // Set transaction status message.
  if ($status = commerce_liqpay_transaction_statuses($transaction->remote_status)) {
    if (!empty($status['message'])) {
      drupal_set_message($status['message']);
    }
  }

  // Redirect user to next or previous page based on transaction status.
  if ($transaction->status == COMMERCE_PAYMENT_STATUS_FAILURE) {
    drupal_goto($path_back);
  }
  else {
    drupal_goto($path_next);
  }
}

/**
 * Data and signature validation callback for module pages.
 */
function commerce_liqpay_post_validate() {
  // Post data could not be empty.
  if (empty($_POST['data']) || empty($_POST['signature'])) {
    return FALSE;
  }

  // Check data that comes from LiqPay.
  $data = commerce_liqpay_payment_data_decode($_POST['data']);
  if (empty($data)) {
    return FALSE;
  }

  // Check drupal commerce order.
  $order = commerce_order_load($data['order_id']);
  if (empty($order)) {
    return FALSE;
  }

  // Check signature.
  $payment_method = commerce_payment_method_instance_load($order->data['payment_method']);
  $signature = commerce_liqpay_generate_signature($_POST['data'], $payment_method['settings']['private_key']);
  if ($_POST['signature'] != urldecode($signature)) {
    return FALSE;
  }

  // Load and check order price and currency.
  $order_wrapper = entity_metadata_wrapper('commerce_order', $order);
  $order_currency = $order_wrapper->commerce_order_total->currency_code->value();
  $order_amount = $order_wrapper->commerce_order_total->amount->value();
  $amount = commerce_currency_amount_to_decimal($order_amount, $order_currency);
  if (($amount != (float) $data['amount']) || ($order_currency != $data['currency'])) {
    return FALSE;
  }

  // Check remote transaction status.
  $status = commerce_liqpay_transaction_statuses($data['status']);
  if (empty($status)) {
    return FALSE;
  }

  // Find out a message for this transaction.
  if (empty($payment_method['settings']['messages']['message_' . $data['status']])) {
    $message = $status['message'];
  }
  else {
    $message = $payment_method['settings']['messages']['message_' . $data['status']];
  }

  // Everything is ok with this request.
  return array(
    'order' => $order,
    'message' => $message,
    'post_data' => $data,
    'remote_status' => $status,
    'payment_method' => $payment_method,
  );
}

/**
 * Create or update commerce payment transaction.
 */
function commerce_liqpay_process_transaction($data) {
  // Prepare commerce transaction object.
  $transaction_conditions = array(
    'payment_method' => 'liqpay',
    'order_id' => $data['order']->order_id,
  );
  $transactions = commerce_payment_transaction_load_multiple(array(), $transaction_conditions);
  if (!empty($transactions)) {
    $transaction = reset($transactions);
    $transaction->revision = TRUE;
  }

  // In case if user trying to pay the same order twice.
  if (empty($transaction) || (!empty($transaction->remote_id) && $transaction->remote_id != $data['post_data']['transaction_id'])) {
    $transaction = commerce_payment_transaction_new('liqpay', $data['order']->order_id);
    $payment_method = commerce_payment_method_instance_load($data['order']->data['payment_method']);
    $transaction->instance_id = $payment_method['instance_id'];
  }

  // Update currency and amount values.
  $transaction->currency_code = $data['post_data']['currency'];
  $transaction->amount = commerce_currency_decimal_to_amount($data['post_data']['amount'], $transaction->currency_code);

  // Set transaction status and message.
  $transaction->remote_id = $data['post_data']['transaction_id'];
  $transaction->status = $data['remote_status']['status'];
  $transaction->remote_status = $data['post_data']['status'];
  $transaction->message = $data['message'];

  // Save or update payment transaction.
  commerce_payment_transaction_save($transaction);
}
